@@ -154,6 +154,8 @@ struct xrdp_client_info
   int client_os_minor;
 
   int no_orders_supported;
+  int use_cache_glyph_v2;
+  int rail_enable;
 };
 
 #endif
