@@ -360,6 +360,33 @@ xrdp_caps_process_brushcache(struct xrdp_rdp *self, struct stream *s,
     return 0;
 }
 
+/*****************************************************************************/
+static int
+xrdp_caps_process_glyphcache(struct xrdp_rdp *self, struct stream *s,
+                             int len)
+{
+    int glyph_support_level;
+
+    if (len < 40 + 4 + 2 + 2) /* MS-RDPBCGR 2.2.7.1.8 */
+    {
+        g_writeln("xrdp_caps_process_glyphcache: error");
+        return 1;
+    }
+
+    in_uint8s(s, 40);  /* glyph cache */
+    in_uint8s(s, 4);   /* frag cache */
+    in_uint16_le(s, glyph_support_level);
+    in_uint8s(s, 2);   /* pad */
+
+    if (glyph_support_level == GLYPH_SUPPORT_ENCODE)
+    {
+        self->client_info.use_cache_glyph_v2 = 1;
+    }
+    g_writeln("xrdp_caps_process_glyphcache: support level %d ",
+              glyph_support_level);
+    return 0;
+}
+
 /*****************************************************************************/
 int
 xrdp_caps_process_offscreen_bmpcache(struct xrdp_rdp *self, struct stream *s,
@@ -610,90 +637,91 @@ xrdp_caps_process_confirm_active(struct xrdp_rdp *self, struct stream *s)
         len -= 4;
         switch (type)
         {
-            case RDP_CAPSET_GENERAL: /* 1 */
-                DEBUG(("RDP_CAPSET_GENERAL"));
+            case CAPSTYPE_GENERAL:
+                DEBUG(("CAPSTYPE_GENERAL"));
                 xrdp_caps_process_general(self, s, len);
                 break;
-            case RDP_CAPSET_BITMAP: /* 2 */
-                DEBUG(("RDP_CAPSET_BITMAP"));
+            case CAPSTYPE_BITMAP:
+                DEBUG(("CAPSTYPE_BITMAP"));
                 break;
-            case RDP_CAPSET_ORDER: /* 3 */
-                DEBUG(("RDP_CAPSET_ORDER"));
+            case CAPSTYPE_ORDER:
+                DEBUG(("CAPSTYPE_ORDER"));
                 xrdp_caps_process_order(self, s, len);
                 break;
-            case RDP_CAPSET_BMPCACHE: /* 4 */
-                DEBUG(("RDP_CAPSET_BMPCACHE"));
+            case CAPSTYPE_BITMAPCACHE:
+                DEBUG(("CAPSTYPE_BMPCACHE"));
                 xrdp_caps_process_bmpcache(self, s, len);
                 break;
-            case RDP_CAPSET_CONTROL: /* 5 */
-                DEBUG(("RDP_CAPSET_CONTROL"));
+            case CAPSTYPE_CONTROL:
+                DEBUG(("CAPSTYPE_CONTROL"));
                 break;
             case 6:
                 xrdp_caps_process_cache_v3_codec_id(self, s, len);
                 break;
-            case RDP_CAPSET_ACTIVATE: /* 7 */
-                DEBUG(("RDP_CAPSET_ACTIVATE"));
+            case CAPSTYPE_ACTIVATION:
+                DEBUG(("CAPSTYPE_ACTIVAION"));
                 break;
-            case RDP_CAPSET_POINTER: /* 8 */
-                DEBUG(("RDP_CAPSET_POINTER"));
+            case CAPSTYPE_POINTER:
+                DEBUG(("CAPSTYPE_POINTER"));
                 xrdp_caps_process_pointer(self, s, len);
                 break;
-            case RDP_CAPSET_SHARE: /* 9 */
-                DEBUG(("RDP_CAPSET_SHARE"));
+            case CAPSTYPE_SHARE:
+                DEBUG(("CAPSTYPE_SHARE"));
                 break;
-            case RDP_CAPSET_COLCACHE: /* 10 */
-                DEBUG(("RDP_CAPSET_COLCACHE"));
+            case CAPSTYPE_COLORCACHE:
+                DEBUG(("CAPSTYPE_COLORCACHE"));
                 break;
-            case 12: /* 12 */
-                DEBUG(("--12"));
+            case CAPSTYPE_SOUND:
+                DEBUG(("CAPSTYPE_SOUND"));
                 break;
-            case 13: /* 13 */
+            case CAPSTYPE_INPUT:
                 xrdp_caps_process_input(self, s, len);
                 break;
-            case 14: /* 14 */
-                DEBUG(("--14"));
+            case CAPSTYPE_FONT:
+                DEBUG(("CAPSTYPE_FONT"));
                 break;
-            case RDP_CAPSET_BRUSHCACHE: /* 15 */
+            case CAPSTYPE_BRUSH:
                 xrdp_caps_process_brushcache(self, s, len);
                 break;
-            case 16: /* 16 */
-                DEBUG(("--16"));
+            case CAPSTYPE_GLYPHCACHE:
+                DEBUG(("CAPSTYPE_GLYPHCACHE"));
+                xrdp_caps_process_glyphcache(self, s, len);
                 break;
-            case 17: /* 17 */
-                DEBUG(("CAPSET_TYPE_OFFSCREEN_CACHE"));
+            case CAPSTYPE_OFFSCREENCACHE:
+                DEBUG(("CAPSTYPE_OFFSCREENCACHE"));
                 xrdp_caps_process_offscreen_bmpcache(self, s, len);
                 break;
-            case RDP_CAPSET_BMPCACHE2: /* 19 */
-                DEBUG(("RDP_CAPSET_BMPCACHE2"));
+            case CAPSTYPE_BITMAPCACHE_REV2:
+                DEBUG(("CAPSTYPE_BITMAPCACHE_REV2"));
                 xrdp_caps_process_bmpcache2(self, s, len);
                 break;
-            case 20: /* 20 */
-                DEBUG(("--20"));
+            case CAPSTYPE_VIRTUALCHANNEL:
+                DEBUG(("CAPSTYPE_VIRTUALCHANNEL"));
                 break;
-            case 21: /* 21 */
-                DEBUG(("--21"));
+            case CAPSTYPE_DRAWNINGRIDCACHE:
+                DEBUG(("CAPSTYPE_DRAWNINGRIDCACHE"));
                 break;
-            case 22: /* 22 */
-                DEBUG(("--22"));
+            case CAPSTYPE_DRAWGDIPLUS:
+                DEBUG(("CAPSTYPE_DRAWGDIPLUS"));
                 break;
-            case 0x0017: /* 23 CAPSETTYPE_RAIL */
+            case CAPSTYPE_RAIL:
                 xrdp_caps_process_rail(self, s, len);
                 break;
-            case 0x0018: /* 24 CAPSETTYPE_WINDOW */
+            case CAPSTYPE_WINDOW:
                 xrdp_caps_process_window(self, s, len);
                 break;
-            case 0x001A: /* 26 CAPSETTYPE_MULTIFRAGMENTUPDATE */
+            case CAPSSETTYPE_MULTIFRAGMENTUPDATE:
                 xrdp_caps_process_multifragmentupdate(self, s, len);
                 break;
-            case RDP_CAPSET_BMPCODECS: /* 0x1d(29) */
+            case CAPSETTYPE_SURFACE_COMMANDS:
+                xrdp_caps_process_surface_cmds(self, s, len);
+                break;
+            case CAPSSETTYPE_BITMAP_CODECS:
                 xrdp_caps_process_codecs(self, s, len);
                 break;
-            case 0x001E: /* CAPSSETTYPE_FRAME_ACKNOWLEDGE */
+            case CAPSTYPE_FRAME_ACKNOWLEDGE:
                 xrdp_caps_process_frame_ack(self, s, len);
                 break;
-            case RDP_CAPSET_SURFCMDS: /* CAPSETTYPE_SURFACE_COMMANDS */
-                xrdp_caps_process_surface_cmds(self, s, len);
-                break;
             default:
                 g_writeln("unknown in xrdp_caps_process_confirm_active %d", type);
                 break;
@@ -757,29 +785,27 @@ xrdp_caps_send_demand_active(struct xrdp_rdp *self)
 
     /* Output share capability set */
     caps_count++;
-    out_uint16_le(s, RDP_CAPSET_SHARE);
-    out_uint16_le(s, RDP_CAPLEN_SHARE);
+    out_uint16_le(s, CAPSTYPE_SHARE);
+    out_uint16_le(s, CAPSTYPE_SHARE_LEN);
     out_uint16_le(s, self->mcs_channel);
     out_uint16_be(s, 0xb5e2); /* 0x73e1 */
 
     /* Output general capability set */
     caps_count++;
-    out_uint16_le(s, RDP_CAPSET_GENERAL); /* 1 */
-    out_uint16_le(s, RDP_CAPLEN_GENERAL); /* 24(0x18) */
-    out_uint16_le(s, 1); /* OS major type */
-    out_uint16_le(s, 3); /* OS minor type */
+    out_uint16_le(s, CAPSTYPE_GENERAL);
+    out_uint16_le(s, CAPSTYPE_GENERAL_LEN);
+    out_uint16_le(s, OSMAJORTYPE_WINDOWS);
+    out_uint16_le(s, OSMINORTYPE_WINDOWS_NT);
     out_uint16_le(s, 0x200); /* Protocol version */
     out_uint16_le(s, 0); /* pad */
     out_uint16_le(s, 0); /* Compression types */
-    /* NO_BITMAP_COMPRESSION_HDR 0x0400
-       FASTPATH_OUTPUT_SUPPORTED 0x0001 */
     if (self->client_info.use_fast_path & 1)
     {
-        out_uint16_le(s, 0x401);
+        out_uint16_le(s, NO_BITMAP_COMPRESSION_HDR | FASTPATH_OUTPUT_SUPPORTED);
     }
     else
     {
-        out_uint16_le(s, 0x400);
+        out_uint16_le(s, NO_BITMAP_COMPRESSION_HDR);
     }
     out_uint16_le(s, 0); /* Update capability */
     out_uint16_le(s, 0); /* Remote unshare capability */
@@ -788,8 +814,8 @@ xrdp_caps_send_demand_active(struct xrdp_rdp *self)
 
     /* Output bitmap capability set */
     caps_count++;
-    out_uint16_le(s, RDP_CAPSET_BITMAP); /* 2 */
-    out_uint16_le(s, RDP_CAPLEN_BITMAP); /* 28(0x1c) */
+    out_uint16_le(s, CAPSTYPE_BITMAP);
+    out_uint16_le(s, CAPSTYPE_BITMAP_LEN);
     out_uint16_le(s, self->client_info.bpp); /* Preferred BPP */
     out_uint16_le(s, 1); /* Receive 1 BPP */
     out_uint16_le(s, 1); /* Receive 4 BPP */
@@ -805,13 +831,13 @@ xrdp_caps_send_demand_active(struct xrdp_rdp *self)
 
     /* Output font capability set */
     caps_count++;
-    out_uint16_le(s, RDP_CAPSET_FONT); /* 14 */
-    out_uint16_le(s, RDP_CAPLEN_FONT); /* 4 */
+    out_uint16_le(s, CAPSTYPE_FONT);
+    out_uint16_le(s, CAPSTYPE_FONT_LEN);
 
     /* Output order capability set */
     caps_count++;
-    out_uint16_le(s, RDP_CAPSET_ORDER); /* 3 */
-    out_uint16_le(s, RDP_CAPLEN_ORDER); /* 88(0x58) */
+    out_uint16_le(s, CAPSTYPE_ORDER);
+    out_uint16_le(s, CAPSTYPE_ORDER_LEN);
     out_uint8s(s, 16);
     out_uint32_be(s, 0x40420f00);
     out_uint16_le(s, 1); /* Cache X granularity */
@@ -863,7 +889,7 @@ xrdp_caps_send_demand_active(struct xrdp_rdp *self)
 
     /* Output bmpcodecs capability set */
     caps_count++;
-    out_uint16_le(s, RDP_CAPSET_BMPCODECS);
+    out_uint16_le(s, CAPSSETTYPE_BITMAP_CODECS);
     codec_caps_size_ptr = s->p;
     out_uint8s(s, 2); /* cap len set later */
     codec_caps_count = 0;
@@ -899,32 +925,36 @@ xrdp_caps_send_demand_active(struct xrdp_rdp *self)
     out_uint8(s, 75); /* jpeg compression ratio */
     /* calculate and set size and count */
     codec_caps_size = (int)(s->p - codec_caps_size_ptr);
-    codec_caps_size += 2; /* 2 bytes for RDP_CAPSET_BMPCODECS above */
+    codec_caps_size += 2; /* 2 bytes for CAPSTYPE_BMPCODECS above */
     codec_caps_size_ptr[0] = codec_caps_size;
     codec_caps_size_ptr[1] = codec_caps_size >> 8;
     codec_caps_count_ptr[0] = codec_caps_count;
 
     /* Output color cache capability set */
     caps_count++;
-    out_uint16_le(s, RDP_CAPSET_COLCACHE);
-    out_uint16_le(s, RDP_CAPLEN_COLCACHE);
+    out_uint16_le(s, CAPSTYPE_COLORCACHE);
+    out_uint16_le(s, CAPSTYPE_COLORCACHE_LEN);
     out_uint16_le(s, 6); /* cache size */
     out_uint16_le(s, 0); /* pad */
 
     /* Output pointer capability set */
     caps_count++;
-    out_uint16_le(s, RDP_CAPSET_POINTER);
-    out_uint16_le(s, RDP_CAPLEN_POINTER);
+    out_uint16_le(s, CAPSTYPE_POINTER);
+    out_uint16_le(s, CAPSTYPE_POINTER_LEN);
     out_uint16_le(s, 1); /* Colour pointer */
     out_uint16_le(s, 0x19); /* Cache size */
     out_uint16_le(s, 0x19); /* Cache size */
 
     /* Output input capability set */
     caps_count++;
-    out_uint16_le(s, RDP_CAPSET_INPUT); /* 13(0xd) */
-    out_uint16_le(s, RDP_CAPLEN_INPUT); /* 88(0x58) */
+    out_uint16_le(s, CAPSTYPE_INPUT);
+    out_uint16_le(s, CAPSTYPE_INPUT_LEN);
+
+    flags = INPUT_FLAG_SCANCODES |
+            INPUT_FLAG_MOUSEX    |
+            INPUT_FLAG_UNICODE   |
+            TS_INPUT_FLAG_MOUSE_HWHEEL;
 
-    flags = INPUT_FLAG_SCANCODES | INPUT_FLAG_MOUSEX | INPUT_FLAG_UNICODE;
     if (self->client_info.use_fast_path & 2)
     {
         flags |= INPUT_FLAG_FASTPATH_INPUT | INPUT_FLAG_FASTPATH_INPUT2;
@@ -932,20 +962,24 @@ xrdp_caps_send_demand_active(struct xrdp_rdp *self)
     out_uint16_le(s, flags);
     out_uint8s(s, 82);
 
-    /* Remote Programs Capability Set */
-    caps_count++;
-    out_uint16_le(s, 0x0017); /* CAPSETTYPE_RAIL */
-    out_uint16_le(s, 8);
-    out_uint32_le(s, 3); /* TS_RAIL_LEVEL_SUPPORTED
-                          TS_RAIL_LEVEL_DOCKED_LANGBAR_SUPPORTED */
+    if (self->client_info.rail_enable) /* MS-RDPERP 3.3.5.1.4 */
+    {
+        /* Remote Programs Capability Set */
+        caps_count++;
+        out_uint16_le(s, CAPSTYPE_RAIL);
+        out_uint16_le(s, 8); /* LengthCapability: MS-RDPERP 2.2.1.1.1 */
+        out_uint32_le(s, 3); /* See: https://msdn.microsoft.com/en-us/library/cc242518.aspx
+                                TS_RAIL_LEVEL_SUPPORTED
+                                TS_RAIL_LEVEL_DOCKED_LANGBAR_SUPPORTED */
 
-    /* Window List Capability Set */
-    caps_count++;
-    out_uint16_le(s, 0x0018); /* CAPSETTYPE_WINDOW */
-    out_uint16_le(s, 11);
-    out_uint32_le(s, 2); /* TS_WINDOW_LEVEL_SUPPORTED_EX */
-    out_uint8(s, 3); /* NumIconCaches */
-    out_uint16_le(s, 12); /* NumIconCacheEntries */
+        /* Window List Capability Set */
+        caps_count++;
+        out_uint16_le(s, CAPSTYPE_WINDOW);
+        out_uint16_le(s, 11); /* LengthCapability: MS-RDPERP 2.2.1.1.2 */
+        out_uint32_le(s, TS_WINDOW_LEVEL_SUPPORTED_EX);
+        out_uint8(s, 3); /* NumIconCaches */
+        out_uint16_le(s, 12); /* NumIconCacheEntries */
+    }
 
     /* 6 - bitmap cache v3 codecid */
     caps_count++;
@@ -957,20 +991,20 @@ xrdp_caps_send_demand_active(struct xrdp_rdp *self)
     {
         /* multifragment update */
         caps_count++;
-        out_uint16_le(s, RDP_CAPSET_MULTIFRAGMENT); /* 26 CAPSETTYPE_MULTIFRAGMENTUPDATE */
-        out_uint16_le(s, RDP_CAPLEN_MULTIFRAGMENT);
+        out_uint16_le(s, CAPSSETTYPE_MULTIFRAGMENTUPDATE);
+        out_uint16_le(s, CAPSSETTYPE_MULTIFRAGMENTUPDATE_LEN);
         out_uint32_le(s, 3 * 1024 * 1024); /* 3MB */
 
         /* frame acks */
         caps_count++;
-        out_uint16_le(s, RDP_CAPSET_FRAME_ACKNOWLEDGE); /* CAPSETTYPE_FRAME_ACKNOWLEDGE */
-        out_uint16_le(s, RDP_CAPLEN_FRAME_ACKNOWLEDGE);
+        out_uint16_le(s, CAPSTYPE_FRAME_ACKNOWLEDGE);
+        out_uint16_le(s, CAPSTYPE_FRAME_ACKNOWLEDGE_LEN);
         out_uint32_le(s, 2); /* 2 frames in flight */
 
         /* surface commands */
         caps_count++;
-        out_uint16_le(s, RDP_CAPSET_SURFCMDS); /* CAPSETTYPE_SURFACE_COMMANDS */
-        out_uint16_le(s, RDP_CAPLEN_SURFCMDS); /* lengthCapability */
+        out_uint16_le(s, CAPSETTYPE_SURFACE_COMMANDS);
+        out_uint16_le(s, CAPSETTYPE_SURFACE_COMMANDS_LEN);
         out_uint32_le(s, (SURFCMDS_SETSURFACEBITS |
                           SURFCMDS_FRAMEMARKER |
                           SURFCMDS_STREAMSUFRACEBITS)); /* cmdFlags */
@@ -990,7 +1024,7 @@ xrdp_caps_send_demand_active(struct xrdp_rdp *self)
     caps_count_ptr[2] = caps_count >> 16;
     caps_count_ptr[3] = caps_count >> 24;
 
-    if (xrdp_rdp_send(self, s, RDP_PDU_DEMAND_ACTIVE) != 0)
+    if (xrdp_rdp_send(self, s, PDUTYPE_DEMANDACTIVEPDU) != 0)
     {
         free_stream(s);
         return 1;
