@@ -43,6 +43,10 @@
 char g_fuse_root_path[256] = "";
 char g_fuse_clipboard_path[256] = ""; /* for clipboard use */
 
+#if defined(HAVE_CONFIG_H)
+#include <config_ac.h>
+#endif
+
 #ifndef XRDP_FUSE
 
 /******************************************************************************
@@ -65,13 +69,13 @@ int xfuse_deinit(void)          { return 0; }
 int xfuse_check_wait_objs(void) { return 0; }
 int xfuse_get_wait_objs(tbus *objs, int *count, int *timeout) { return 0; }
 int xfuse_clear_clip_dir(void)  { return 0; }
-int xfuse_file_contents_range(int stream_id, char *data, int data_bytes)     { return 0; }
+int xfuse_file_contents_range(int stream_id, const char *data, int data_bytes)     { return 0; }
 int xfuse_file_contents_size(int stream_id, int file_size)                   { return 0; }
-int xfuse_add_clip_dir_item(char *filename, int flags, int size, int lindex) { return 0; }
-int xfuse_create_share(tui32 device_id, char *dirname)                       { return 0; }
+int xfuse_add_clip_dir_item(const char *filename, int flags, int size, int lindex) { return 0; }
+int xfuse_create_share(tui32 device_id, const char *dirname)                       { return 0; }
 void xfuse_devredir_cb_open_file(void *vp, tui32 IoStatus, tui32 DeviceId, tui32 FileId)     {}
-void xfuse_devredir_cb_write_file(void *vp, char *buf, size_t length)        {}
-void xfuse_devredir_cb_read_file(void *vp, char *buf, size_t length)         {}
+void xfuse_devredir_cb_write_file(void *vp, const char *buf, size_t length)        {}
+void xfuse_devredir_cb_read_file(void *vp, const char *buf, size_t length)         {}
 int  xfuse_devredir_cb_enum_dir(void *vp, struct xrdp_inode *xinode)         { return 0; }
 void xfuse_devredir_cb_enum_dir_done(void *vp, tui32 IoStatus)               {}
 void xfuse_devredir_cb_rmdir_or_file(void *vp, tui32 IoStatus)               {}
@@ -273,9 +277,9 @@ static void fuse_reverse_pathname(char *full_path, char *reverse_path);
 static struct xrdp_inode * xfuse_get_inode_from_pinode_name(fuse_ino_t pinode,
                                                             const char *name);
 
-static struct xrdp_inode * xfuse_create_file_in_xrdp_fs(tui32 device_id,
-                                                        int pinode, char *name,
-                                                        int type);
+static struct xrdp_inode *
+xfuse_create_file_in_xrdp_fs(tui32 device_id, int pinode, const char *name,
+                             int type);
 
 static int  xfuse_does_file_exist(fuse_ino_t parent, char *name);
 static int  xfuse_delete_file_with_xinode(XRDP_INODE *xinode);
@@ -361,7 +365,7 @@ static void xfuse_mark_as_stale(fuse_ino_t pinode);
 static void xfuse_delete_stale_entries(fuse_ino_t pinode);
 
 /*****************************************************************************/
-int APP_CC
+int
 load_fuse_config(void)
 {
     int index;
@@ -403,7 +407,7 @@ load_fuse_config(void)
  * @return 0 on success, -1 on failure
  *****************************************************************************/
 
-int APP_CC
+int
 xfuse_init(void)
 {
     struct fuse_args args = FUSE_ARGS_INIT(0, NULL);
@@ -485,7 +489,7 @@ xfuse_init(void)
  * @return 0 on success, -1 on failure
  *****************************************************************************/
 
-int APP_CC
+int
 xfuse_deinit(void)
 {
     xfuse_deinit_xrdp_fs();
@@ -596,7 +600,7 @@ int xfuse_get_wait_objs(tbus *objs, int *count, int *timeout)
  * @return 0 on success, -1 on failure
  *****************************************************************************/
 
-int xfuse_create_share(tui32 device_id, char *dirname)
+int xfuse_create_share(tui32 device_id, const char *dirname)
 {
 #if 0
     XFUSE_INFO  *fip;
@@ -607,6 +611,12 @@ int xfuse_create_share(tui32 device_id, char *dirname)
     if (dirname == NULL || strlen(dirname) == 0)
         return -1;
 
+    /* Do we have an inode table yet? */
+    if (xfuse_init_xrdp_fs())
+    {
+        return -1;
+    }
+
     xinode = g_new0(struct xrdp_inode, 1);
     if (xinode == NULL)
     {
@@ -709,7 +719,8 @@ int xfuse_clear_clip_dir(void)
  * @return 0 on success, -1 on failure
  *****************************************************************************/
 
-int xfuse_file_contents_range(int stream_id, char *data, int data_bytes)
+int
+xfuse_file_contents_range(int stream_id, const char *data, int data_bytes)
 {
     log_debug("entered: stream_id=%d data_bytes=%d", stream_id, data_bytes);
 
@@ -756,7 +767,8 @@ int xfuse_file_contents_range(int stream_id, char *data, int data_bytes)
  * @return 0 on success, -1 on failure
  *****************************************************************************/
 
-int xfuse_add_clip_dir_item(char *filename, int flags, int size, int lindex)
+int
+xfuse_add_clip_dir_item(const char *filename, int flags, int size, int lindex)
 {
     log_debug("entered: filename=%s flags=%d size=%d lindex=%d",
               filename, flags, size, lindex);
@@ -848,10 +860,16 @@ static int xfuse_init_lib(struct fuse_args *args)
  *
  *****************************************************************************/
 
-static int xfuse_init_xrdp_fs()
+static int xfuse_init_xrdp_fs(void)
 {
     struct xrdp_inode *xino;
 
+    /* Already called? */
+    if (g_xrdp_fs.inode_table != NULL)
+    {
+        return 0;
+    }
+
     g_xrdp_fs.inode_table = g_new0(struct xrdp_inode *, 4096);
     if (g_xrdp_fs.inode_table == NULL)
     {
@@ -949,7 +967,7 @@ static int xfuse_init_xrdp_fs()
  * @return 0 on success, -1 on failure
  *****************************************************************************/
 
-static int xfuse_deinit_xrdp_fs()
+static int xfuse_deinit_xrdp_fs(void)
 {
     return 0;
 }
@@ -1043,7 +1061,7 @@ static void xfuse_create_file(fuse_req_t req, fuse_ino_t parent,
 }
 #endif
 
-static void xfuse_dump_fs()
+static void xfuse_dump_fs(void)
 {
     fuse_ino_t i;
     struct xrdp_inode *xinode;
@@ -1204,16 +1222,23 @@ static struct xrdp_inode * xfuse_get_inode_from_pinode_name(fuse_ino_t pinode,
  * @return XRDP_INODE on success, NULL on failure
  *****************************************************************************/
 
-static struct xrdp_inode * xfuse_create_file_in_xrdp_fs(tui32 device_id,
-                                                        int pinode, char *name,
-                                                        int type)
+static struct xrdp_inode *
+xfuse_create_file_in_xrdp_fs(tui32 device_id, int pinode, const char *name,
+                             int type)
 {
     XRDP_INODE *xinode;
     XRDP_INODE *xinodep;
+    time_t cur_time;
 
     if ((name == NULL) || (strlen(name) == 0))
         return NULL;
 
+    /* Do we have an inode table yet? */
+    if (xfuse_init_xrdp_fs())
+    {
+        return NULL;
+    }
+
     xinode = g_new0(XRDP_INODE, 1);
     if (xinode == NULL)
     {
@@ -1221,14 +1246,16 @@ static struct xrdp_inode * xfuse_create_file_in_xrdp_fs(tui32 device_id,
         return NULL;
     }
 
+    cur_time = time(0);
+
     xinode->parent_inode = pinode;
     xinode->inode = g_xrdp_fs.next_node++;
     xinode->nlink = 1;
     xinode->uid = getuid();
     xinode->gid = getgid();
-    xinode->atime = time(0);
-    xinode->mtime = time(0);
-    xinode->ctime = time(0);
+    xinode->atime = cur_time;
+    xinode->mtime = cur_time;
+    xinode->ctime = cur_time;
     xinode->device_id = device_id;
     xinode->is_synced = 1;
     strcpy(xinode->name, name);
@@ -1387,7 +1414,7 @@ static int xfuse_recursive_delete_dir_with_xinode(XRDP_INODE *xinode)
     return 0;
 }
 
-static void xfuse_update_xrdpfs_size()
+static void xfuse_update_xrdpfs_size(void)
 {
     void *vp;
     int diff;
@@ -1681,7 +1708,7 @@ done:
     free(fip);
 }
 
-void xfuse_devredir_cb_read_file(void *vp, char *buf, size_t length)
+void xfuse_devredir_cb_read_file(void *vp, const char *buf, size_t length)
 {
     XFUSE_INFO   *fip;
 
@@ -1696,7 +1723,7 @@ void xfuse_devredir_cb_read_file(void *vp, char *buf, size_t length)
     free(fip);
 }
 
-void xfuse_devredir_cb_write_file(void *vp, char *buf, size_t length)
+void xfuse_devredir_cb_write_file(void *vp, const char *buf, size_t length)
 {
     XRDP_INODE   *xinode;
     XFUSE_INFO   *fip;
