@@ -22,6 +22,10 @@
  * CLIPRDR_FILEDESCRIPTOR
  * http://msdn.microsoft.com/en-us/library/ff362447%28prot.20%29.aspx */
 
+#if defined(HAVE_CONFIG_H)
+#include <config_ac.h>
+#endif
+
 #include <sys/time.h>
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
@@ -103,7 +107,7 @@ static int g_file_request_sent_type = 0;
 
 /*****************************************************************************/
 #if 0
-static tui64 APP_CC
+static tui64
 timeval2wintime(struct timeval *tv)
 {
     tui64 result;
@@ -119,7 +123,7 @@ timeval2wintime(struct timeval *tv)
 /*****************************************************************************/
 /* this will replace %20 or any hex with the space or correct char
  * returns error */
-static int APP_CC
+static int
 clipboard_check_file(char *filename)
 {
     char lfilename[256];
@@ -154,8 +158,8 @@ clipboard_check_file(char *filename)
 }
 
 /*****************************************************************************/
-static int APP_CC
-clipboard_get_file(char* file, int bytes)
+static int
+clipboard_get_file(const char *file, int bytes)
 {
     int sindex;
     int pindex;
@@ -232,8 +236,8 @@ clipboard_get_file(char* file, int bytes)
 }
 
 /*****************************************************************************/
-static int APP_CC
-clipboard_get_files(char *files, int bytes)
+static int
+clipboard_get_files(const char *files, int bytes)
 {
     int index;
     int file_index;
@@ -274,8 +278,8 @@ clipboard_get_files(char *files, int bytes)
 /*****************************************************************************/
 /* server to client */
 /* response to client asking for clipboard contents that is file list */
-int APP_CC
-clipboard_send_data_response_for_file(char *data, int data_size)
+int
+clipboard_send_data_response_for_file(const char *data, int data_size)
 {
     struct stream *s;
     int size;
@@ -340,7 +344,7 @@ clipboard_send_data_response_for_file(char *data, int data_size)
 
 /*****************************************************************************/
 /* send the file size from server to the client */
-static int APP_CC
+static int
 clipboard_send_file_size(int streamId, int lindex)
 {
     struct stream *s;
@@ -381,7 +385,7 @@ clipboard_send_file_size(int streamId, int lindex)
 
 /*****************************************************************************/
 /* ask the client to send the file size */
-int APP_CC
+int
 clipboard_request_file_size(int stream_id, int lindex)
 {
     struct stream *s;
@@ -417,7 +421,7 @@ clipboard_request_file_size(int stream_id, int lindex)
 
 /*****************************************************************************/
 /* send a chunk of the file from server to client */
-static int APP_CC
+static int
 clipboard_send_file_data(int streamId, int lindex,
                          int nPositionLow, int cbRequested)
 {
@@ -484,7 +488,7 @@ clipboard_send_file_data(int streamId, int lindex,
 
 /*****************************************************************************/
 /* ask the client to send the file size */
-int APP_CC
+int
 clipboard_request_file_data(int stream_id, int lindex, int offset,
                             int request_bytes)
 {
@@ -524,7 +528,7 @@ clipboard_request_file_data(int stream_id, int lindex, int offset,
 
 /*****************************************************************************/
 /* client is asking from info about a file */
-int APP_CC
+int
 clipboard_process_file_request(struct stream *s, int clip_msg_status,
                                int clip_msg_len)
 {
@@ -558,7 +562,7 @@ clipboard_process_file_request(struct stream *s, int clip_msg_status,
 /*****************************************************************************/
 /* server requested info about the file and this is the response
    it's either the file size or file data */
-int APP_CC
+int
 clipboard_process_file_response(struct stream *s, int clip_msg_status,
                                 int clip_msg_len)
 {
@@ -591,7 +595,7 @@ clipboard_process_file_response(struct stream *s, int clip_msg_status,
 
 /*****************************************************************************/
 /* read in CLIPRDR_FILEDESCRIPTOR */
-static int APP_CC
+static int
 clipboard_c2s_in_file_info(struct stream *s, struct clip_file_desc *cfd)
 {
     int num_chars;
@@ -623,7 +627,7 @@ clipboard_c2s_in_file_info(struct stream *s, struct clip_file_desc *cfd)
 }
 
 /*****************************************************************************/
-int APP_CC
+int
 clipboard_c2s_in_files(struct stream *s, char *file_list)
 {
     int cItems;
