@@ -38,6 +38,12 @@
 #if defined(XRDP_FDK_AAC)
 #include <fdk-aac/aacenc_lib.h>
 static HANDLE_AACENCODER g_fdk_aac_encoder = 0;
+
+#define AACENCODER_LIB_VER_GTEQ(vl0, vl1, vl2) \
+    (defined(AACENCODER_LIB_VL0) && \
+        ((AACENCODER_LIB_VL0 > vl0) || \
+         (AACENCODER_LIB_VL0 == vl0 && AACENCODER_LIB_VL1 >= vl1) || \
+         (AACENCODER_LIB_VL0 == vl0 && AACENCODER_LIB_VL1 == vl1 && AACENCODER_LIB_VL2 > vl2)))
 #endif
 
 #if defined(XRDP_OPUS)
@@ -400,19 +406,16 @@ sound_process_output_format(int aindex, int wFormatTag, int nChannels,
             LOG(0, ("wFormatTag, fdk aac"));
             g_client_does_fdk_aac = 1;
             g_client_fdk_aac_index = aindex;
-            g_bbuf_size = 4096;
             break;
         case WAVE_FORMAT_MPEGLAYER3:
             LOG(0, ("wFormatTag, mp3"));
             g_client_does_mp3lame = 1;
             g_client_mp3lame_index = aindex;
-            g_bbuf_size = 11520;
             break;
         case WAVE_FORMAT_OPUS:
             LOG(0, ("wFormatTag, opus"));
             g_client_does_opus = 1;
             g_client_opus_index = aindex;
-            g_bbuf_size = 11520;
             break;
     }
 
@@ -601,7 +604,12 @@ sound_wave_compress_fdk_aac(char *data, int data_bytes, int *format_index)
         LOG(0, ("    inBufFillLevel %d", info.inBufFillLevel));
         LOG(0, ("    inputChannels %d", info.inputChannels));
         LOG(0, ("    frameLength %d", info.frameLength));
+#if AACENCODER_LIB_VER_GTEQ(4, 0, 0)
+        LOG(0, ("    nDelay %d", info.nDelay));
+        LOG(0, ("    nDelayCore %d", info.nDelayCore));
+#else
         LOG(0, ("    encoderDelay %d", info.encoderDelay));
+#endif
         LOG(0, ("    confBuf"));
         LOG(0, ("    confSize %d", info.confSize));
     }
@@ -840,14 +848,17 @@ sound_wave_compress(char *data, int data_bytes, int *format_index)
 {
     if (g_client_does_fdk_aac)
     {
+        g_bbuf_size = 4096;
         return sound_wave_compress_fdk_aac(data, data_bytes, format_index);
     }
     else if (g_client_does_opus)
     {
+        g_bbuf_size = 11520;
         return sound_wave_compress_opus(data, data_bytes, format_index);
     }
     else if (g_client_does_mp3lame)
     {
+        g_bbuf_size = 11520;
         return sound_wave_compress_mp3lame(data, data_bytes, format_index);
     }
     return data_bytes;
