@@ -26,6 +26,8 @@
 #define FORMAT_TO_BYTES(_format) \
     (_format) == 32 ? sizeof(long) : (_format) / 8
 
+typedef void (*x_server_fatal_cb_type)(void);
+
 int
 xcommon_get_local_time(void);
 int
@@ -34,5 +36,7 @@ int
 xcommon_get_wait_objs(tbus* objs, int* count, int* timeout);
 int
 xcommon_check_wait_objs(void);
+void
+xcommon_set_x_server_fatal_handler(x_server_fatal_cb_type handler);
 
 #endif
