@@ -39,6 +39,7 @@
 #define SESMAN_CFG_USERWM            "UserWindowManager"
 #define SESMAN_CFG_MAX_SESSION       "MaxSessions"
 #define SESMAN_CFG_AUTH_FILE_PATH    "AuthFilePath"
+#define SESMAN_CFG_RECONNECT_SH      "ReconnectScript"
 
 #define SESMAN_CFG_RDP_PARAMS        "X11rdp"
 #define SESMAN_CFG_XORG_PARAMS       "Xorg"
@@ -198,12 +199,17 @@ struct config_sesman
    * @var default_wm
    * @brief Default window manager
    */
-  char default_wm[32];
+  char *default_wm;
   /**
    * @var user_wm
    * @brief Default window manager
    */
   char user_wm[32];
+  /**
+   * @var reconnect_sh
+   * @brief Script executed when reconnected
+   */
+  char *reconnect_sh;
   /**
    * @var auth_file_path
    * @brief Auth file path
@@ -276,20 +282,6 @@ int
 config_read_globals(int file, struct config_sesman* cf,
                     struct list* param_n, struct list* param_v);
 
-/**
- *
- * @brief Reads sesman [logging] configuration section
- * @param file configuration file descriptor
- * @param lc pointer to a log_config struct
- * @param param_n parameter name list
- * @param param_v parameter value list
- * @return 0 on success, 1 on failure
- *
- */
-int
-config_read_logging(int file, struct log_config* lc, struct list* param_n,
-                    struct list* param_v);
-
 /**
  *
  * @brief Reads sesman [Security] configuration section
@@ -334,7 +326,7 @@ config_read_rdp_params(int file, struct config_sesman* cs, struct list* param_n,
 
 /**
  *
- * @brief Reads sesman [XOrg] configuration section
+ * @brief Reads sesman [Xorg] configuration section
  * @param file configuration file descriptor
  * @param cs pointer to a config_sesman struct
  * @param param_n parameter name list
@@ -363,6 +355,14 @@ config_read_vnc_params(int file, struct config_sesman* cs, struct list* param_n,
 int
 config_read_session_variables(int file, struct config_sesman *cs,
                               struct list *param_n, struct list *param_v);
+/**
+ *
+ * @brief Dumps configuration
+ * @param pointer to a config_sesman struct
+ *
+ */
+void
+config_dump(struct config_sesman *config);
 
 void
 config_free(struct config_sesman *cs);
